@@ -1,6 +1,7 @@
 # -*- coding:utf-8 -*-
 #
-# Copyright (C) 2019-2020, Maximilian Köhl <koehl@cs.uni-saarland.de>
+# Copyright (C) 2019-2021, Saarland University
+# Copyright (C) 2019-2021, Maximilian Köhl <koehl@cs.uni-saarland.de>
 
 from __future__ import annotations
 
@@ -14,43 +15,61 @@ import fractions
 import math
 import warnings
 
-from . import errors, operators, properties, types
+from . import errors, operators, types
 
 if t.TYPE_CHECKING:
     from . import context, distributions
 
 
-class Expression(properties.Property, abc.ABC):
+class Expression(abc.ABC):
+    """
+    Abstract base class for expressions.
+    """
+
+    @abc.abstractmethod
+    def infer_type(self, scope: context.Scope) -> types.Type:
+        raise NotImplementedError()
+
+    def infer_target_type(self, scope: context.Scope) -> types.Type:
+        raise errors.InvalidTypeError(
+            f"expression {self} is not a valid assignment target"
+        )
+
     @property
     @abc.abstractmethod
     def children(self) -> t.Sequence[Expression]:
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def is_constant_in(self, scope: context.Scope) -> bool:
         """
-        Returns `True` only if the expression has a constant value in the given scope.
-
-        Arguments:
-            scope: The scope to use.
+        The direct children of the expression.
         """
         raise NotImplementedError()
 
     def traverse(self) -> t.Iterator[Expression]:
+        """
+        Returns an iterator over the subexpressions.
+        """
         yield self
         for child in self.children:
             yield from child.traverse()
 
     @property
     def subexpressions(self) -> t.AbstractSet[Expression]:
+        """
+        A set of subexpressions.
+        """
         return frozenset(self.traverse())
 
     @property
     def is_sampling_free(self) -> bool:
+        """
+        Returns whether the expression is *sampling-free*.
+        """
         return all(not isinstance(e, Sample) for e in self.traverse())
 
     @property
     def used_names(self) -> t.AbstractSet[Name]:
+        """
+        Returns a set of *name expressions* occuring in the expression.
+        """
         return frozenset(child for child in self.traverse() if isinstance(child, Name))
 
 
@@ -61,12 +80,20 @@ class _Leaf(Expression):
 
 
 class Constant(_Leaf, abc.ABC):
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return True
+    pass
 
 
 @d.dataclass(frozen=True)
 class BooleanConstant(Constant):
+    """
+    A boolean constant.
+
+    Attributes
+    ----------
+    boolean:
+        The value of the boolean constant.
+    """
+
     boolean: bool
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -74,14 +101,28 @@ class BooleanConstant(Constant):
 
 
 class NumericConstant(Constant, abc.ABC):
+    """
+    Abstract base class for numeric constants.
+    """
+
     @property
     @abc.abstractmethod
     def as_float(self) -> float:
+        """
+        The numeric constant as a floating-point number.
+
+        Note that this may result in a loss of precision.
+        """
         raise NotImplementedError()
 
     @property
     @abc.abstractmethod
     def as_fraction(self) -> fractions.Fraction:
+        """
+        The numeric constant as a fraction.
+
+        Note that this may result in a loss of precision
+        """
         raise NotImplementedError()
 
 
@@ -91,6 +132,15 @@ FALSE: Expression = BooleanConstant(False)
 
 @d.dataclass(frozen=True)
 class IntegerConstant(NumericConstant):
+    """
+    An integer constant.
+
+    Attributes
+    ----------
+    integer:
+        The value of the integer constant.
+    """
+
     integer: int
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -112,8 +162,22 @@ _NAMED_REAL_MAP: t.Dict[str, NamedReal] = {}
 
 
 class NamedReal(enum.Enum):
+    """
+    An enum of named reals.
+
+    Attributes
+    ----------
+    symbol:
+        The mathematical symbol of the real.
+    float_value:
+        A floating-point approximation of the real.
+    """
+
     PI = "π", math.pi
+    """ The number π. """
+
     E = "e", math.e
+    """ The number e. """
 
     symbol: str
     float_value: float
@@ -129,6 +193,15 @@ Real = t.Union[NamedReal, fractions.Fraction]
 
 @d.dataclass(frozen=True)
 class RealConstant(NumericConstant):
+    """
+    A real constant.
+
+    Attributes
+    ----------
+    real:
+        The real (either :class:`NamedReal` or a fraction).
+    """
+
     real: Real
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -156,19 +229,41 @@ class RealConstant(NumericConstant):
 
 @d.dataclass(frozen=True)
 class Name(_Leaf):
-    identifier: str
+    """
+    A name expression.
 
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return scope.lookup(self.identifier).is_constant_in(scope)
+    Attributes
+    ----------
+    identifier:
+        The identifier.
+    """
+
+    identifier: str
 
     def infer_type(self, scope: context.Scope) -> types.Type:
         return scope.lookup(self.identifier).typ
 
+    def infer_target_type(self, scope: context.Scope) -> types.Type:
+        return self.infer_type(scope)
+
 
 # XXX: this class should be abstract, however, then it would not type-check
 # https://github.com/python/mypy/issues/5374
 @d.dataclass(frozen=True)
 class BinaryExpression(Expression):
+    """
+    Abstract base class for binary expressions.
+
+    Attributes
+    ----------
+    operator:
+        The binary operator (:class:`~momba.model.operators.BinaryOperator`).
+    left:
+        The left operand.
+    right:
+        The right operand.
+    """
+
     operator: operators.BinaryOperator
     left: Expression
     right: Expression
@@ -177,15 +272,21 @@ class BinaryExpression(Expression):
     def children(self) -> t.Sequence[Expression]:
         return self.left, self.right
 
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return self.left.is_constant_in(scope) and self.right.is_constant_in(scope)
-
     # XXX: this method shall be implemented by all subclasses
     def infer_type(self, scope: context.Scope) -> types.Type:
         raise NotImplementedError()
 
 
 class Boolean(BinaryExpression):
+    """
+    A boolean binary expression.
+
+    Attributes
+    ----------
+    operator:
+        The boolean operator (:class:`~momba.model.operators.BooleanOperator`).
+    """
+
     operator: operators.BooleanOperator
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -206,6 +307,15 @@ _REAL_RESULT_OPERATORS = {
 
 
 class ArithmeticBinary(BinaryExpression):
+    """
+    An arithmetic binary expression.
+
+    Attributes
+    ----------
+    operator:
+        The arithmetic operator (:class:`~momba.model.operators.ArithmeticBinaryOperator`).
+    """
+
     operator: operators.ArithmeticBinaryOperator
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -227,6 +337,15 @@ class ArithmeticBinary(BinaryExpression):
 
 
 class Equality(BinaryExpression):
+    """
+    An equality binary expression.
+
+    Attributes
+    ----------
+    operator:
+        The equality operator (:class:`~momba.model.operators.EqualityOperator`).
+    """
+
     operator: operators.EqualityOperator
 
     def get_common_type(self, scope: context.Scope) -> types.Type:
@@ -255,6 +374,15 @@ class Equality(BinaryExpression):
 
 
 class Comparison(BinaryExpression):
+    """
+    A comparison expression.
+
+    Attributes
+    ----------
+    operator:
+        The comparison operator (:class:`~momba.model.operators.ComparisonOperator`).
+    """
+
     operator: operators.ComparisonOperator
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -269,6 +397,19 @@ class Comparison(BinaryExpression):
 
 @d.dataclass(frozen=True)
 class Conditional(Expression):
+    """
+    A ternary conditional expression.
+
+    Attributes
+    ----------
+    condition:
+        The condition.
+    consequence:
+        The consequence to be evaluated if the condition is true.
+    alternative:
+        The alternative to be evaluated if the condition is false.
+    """
+
     condition: Expression
     consequence: Expression
     alternative: Expression
@@ -277,13 +418,6 @@ class Conditional(Expression):
     def children(self) -> t.Sequence[Expression]:
         return self.condition, self.consequence, self.alternative
 
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return (
-            self.condition.is_constant_in(scope)
-            and self.consequence.is_constant_in(scope)
-            and self.alternative.is_constant_in(scope)
-        )
-
     def infer_type(self, scope: context.Scope) -> types.Type:
         condition_type = scope.get_type(self.condition)
         if condition_type != types.BOOL:
@@ -305,7 +439,18 @@ class Conditional(Expression):
 # XXX: this class should be abstract, however, then it would not type-check
 # https://github.com/python/mypy/issues/5374
 @d.dataclass(frozen=True)
-class UnaryExpression(Expression):
+class UnaryExpression(Expression, abc.ABC):
+    """
+    Base class of all unary expressions.
+
+    Attributes
+    ----------
+    operator:
+        The unary operator (:class:`~momba.model.operators.UnaryOperator`).
+    operand:
+        The operand.
+    """
+
     operator: operators.UnaryOperator
     operand: Expression
 
@@ -313,15 +458,21 @@ class UnaryExpression(Expression):
     def children(self) -> t.Sequence[Expression]:
         return (self.operand,)
 
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return self.operand.is_constant_in(scope)
-
     # XXX: this method shall be implemented by all subclasses
     def infer_type(self, scope: context.Scope) -> types.Type:
         raise NotImplementedError()
 
 
 class ArithmeticUnary(UnaryExpression):
+    """
+    An arithmetic unary expression.
+
+    Attributes
+    ----------
+    operator:
+        The arithmetic operator (:class:`~momba.model.operators.ArithmeticUnaryOperator`).
+    """
+
     operator: operators.ArithmeticUnaryOperator
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -334,6 +485,15 @@ class ArithmeticUnary(UnaryExpression):
 
 
 class Not(UnaryExpression):
+    """
+    Logical negation.
+
+    Attributes
+    ----------
+    operator:
+        The logical negation operator (:class:`~momba.model.operators.NotOperator`).
+    """
+
     operator: operators.NotOperator
 
     def infer_type(self, scope: context.Scope) -> types.Type:
@@ -347,6 +507,18 @@ class Not(UnaryExpression):
 
 @d.dataclass(frozen=True)
 class Sample(Expression):
+    """
+    A sample expression.
+
+    Attributes
+    ----------
+    distribution:
+        The type of the distribution to sample from
+        (:class:`~momba.model.distributions.DistributionType`).
+    arguments:
+        The arguments to the distribution.
+    """
+
     distribution: distributions.DistributionType
     arguments: t.Sequence[Expression]
 
@@ -361,9 +533,6 @@ class Sample(Expression):
     def children(self) -> t.Sequence[Expression]:
         return self.arguments
 
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return False
-
     def infer_type(self, scope: context.Scope) -> types.Type:
         # we already know that the arity of the parameters and arguments match
         for argument, parameter_type in zip(
@@ -381,19 +550,27 @@ class Sample(Expression):
 # requires JANI extension `nondet-selection`
 @d.dataclass(frozen=True)
 class Selection(Expression):
-    name: str
+    """
+    A non-deterministic selection expression.
+
+    Attributes
+    ----------
+    variable:
+        The identifier to select over.
+    condition:
+        The condition that should be satisfied.
+    """
+
+    variable: str
     condition: Expression
 
     def infer_type(self, scope: context.Scope) -> types.Type:
-        condition_type = scope.get_type(self.condition)
+        condition_scope = scope.create_child_scope()
+        condition_scope.declare_variable(self.variable, typ=types.REAL)
+        condition_type = condition_scope.get_type(self.condition)
         if condition_type != types.BOOL:
             raise errors.InvalidTypeError("condition must have type `types.BOOL`")
-        declaration = scope.lookup(self.name)
-        assert isinstance(declaration, context.VariableDeclaration)
-        return declaration.typ
-
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return False
+        return types.REAL
 
     @property
     def children(self) -> t.Sequence[Expression]:
@@ -402,19 +579,152 @@ class Selection(Expression):
 
 @d.dataclass(frozen=True)
 class Derivative(Expression):
+    """
+    Derivative of a continuous variable.
+
+    Attributes
+    ==========
+    identifier:
+        The continuous variable.
+    """
+
     identifier: str
 
     def infer_type(self, scope: context.Scope) -> types.Type:
         return types.REAL
 
-    def is_constant_in(self, scope: context.Scope) -> bool:
-        return False
-
     @property
     def children(self) -> t.Sequence[Expression]:
         return ()
 
 
+@d.dataclass(frozen=True)
+class ArrayAccess(Expression):
+    """
+    An array access expression.
+
+    Attributes
+    ----------
+    array:
+        The array to access.
+    index:
+        The index where to access the array.
+    """
+
+    array: Expression
+    index: Expression
+
+    @property
+    def children(self) -> t.Sequence[Expression]:
+        return self.array, self.index
+
+    def infer_type(self, scope: context.Scope) -> types.Type:
+        array_type = scope.get_type(self.array)
+        # TODO: check the type of the index
+        if isinstance(array_type, types.ArrayType):
+            return array_type.element
+        else:
+            raise errors.InvalidTypeError("array of array access must have array type")
+
+    def infer_target_type(self, scope: context.Scope) -> types.Type:
+        return self.infer_type(scope)
+
+
+@d.dataclass(frozen=True)
+class ArrayValue(Expression):
+    """
+    An array value expression.
+
+    Attributes
+    ----------
+    elements:
+        The elements of the array to construct.
+    """
+
+    elements: t.Tuple[Expression, ...]
+
+    def __post_init__(self) -> None:
+        if not self.elements:
+            raise errors.ModelingError("array value expression needs to have elements")
+
+    @property
+    def children(self) -> t.Sequence[Expression]:
+        return self.elements
+
+    def infer_type(self, scope: context.Scope) -> types.Type:
+        common_type: t.Optional[types.Type] = None
+        for element in self.elements:
+            element_type = scope.get_type(element)
+            if common_type is None:
+                common_type = element_type
+            elif element_type.is_assignable_from(common_type):
+                common_type = element_type
+            elif not common_type.is_assignable_from(element_type):
+                raise errors.InvalidTypeError(
+                    "element types are not assignable to a common type"
+                )
+        assert common_type is not None
+        return types.array_of(common_type)
+
+
+@d.dataclass(frozen=True)
+class ArrayConstructor(Expression):
+    """
+    An array constructor expression.
+
+    Attributes
+    ----------
+    variable:
+        The identifier to range over.
+    length:
+        The length of the array.
+    expression:
+        The expression to compute the elements of the array.
+    """
+
+    variable: str
+    length: Expression
+    expression: Expression
+
+    @property
+    def children(self) -> t.Sequence[Expression]:
+        return self.length, self.expression
+
+    def _create_scope(self, scope: context.Scope) -> context.Scope:
+        child_scope = scope.create_child_scope()
+        child_scope.declare_constant(self.variable, types.INT)
+        return child_scope
+
+    def infer_type(self, scope: context.Scope) -> types.Type:
+        if not types.INT.is_assignable_from(scope.get_type(self.length)):
+            raise errors.InvalidTypeError(
+                "length of array constructor has to be an integer"
+            )
+        return types.array_of(self._create_scope(scope).get_type(self.expression))
+
+
+class Trigonometric(UnaryExpression):
+    """
+    A trigonometric expression.
+
+    Attributes
+    ----------
+    operator:
+        The trigonometric function to apply
+        (:class:`~momba.model.operators.TrigonometricFunction`).
+    """
+
+    operator: operators.TrigonometricFunction
+
+    def infer_type(self, scope: context.Scope) -> types.Type:
+        operand_type = scope.get_type(self.operand)
+        if not operand_type.is_numeric:
+            raise errors.InvalidTypeError(
+                "expected numeric type for operand of trigonometric function"
+            )
+        return types.REAL
+
+
 RealValue = t.Union[t.Literal["π", "e"], float, fractions.Fraction, decimal.Decimal]
 NumericValue = t.Union[int, RealValue]
 Value = t.Union[bool, NumericValue]
@@ -423,10 +733,19 @@ ValueOrExpression = t.Union[Expression, Value]
 
 
 class ConversionError(ValueError):
-    pass
+    """
+    Unable to convert the provided value.
+    """
 
 
 def ensure_expr(value_or_expression: ValueOrExpression) -> Expression:
+    """
+    Takes a Python value or expression and returns an expression.
+
+    Implicitly converts floats, integers, and booleans to constant expressions.
+
+    Raises :class:`~momba.model.expressions.ConversionError` if the conversion fails.
+    """
     if isinstance(value_or_expression, Expression):
         return value_or_expression
     elif isinstance(value_or_expression, bool):
@@ -450,6 +769,9 @@ def ite(
     consequence: ValueOrExpression,
     alternative: ValueOrExpression,
 ) -> Expression:
+    """
+    Constructs a conditional expression implicitly converting the arguments.
+    """
     return Conditional(
         ensure_expr(condition), ensure_expr(consequence), ensure_expr(alternative)
     )
@@ -473,93 +795,168 @@ def _boolean_binary_expression(
     return result
 
 
+def logic_not(operand: ValueOrExpression) -> Expression:
+    """
+    Constructs a logical negation expression.
+    """
+    return Not(operators.NotOperator.NOT, ensure_expr(operand))
+
+
+def logic_any(*expressions: ValueOrExpression) -> Expression:
+    """
+    Constructs a disjunction over the provided expressions.
+
+    Returns :attr:`~momba.model.expressions.FALSE` if there are no expressions.
+    """
+    if len(expressions) == 0:
+        return BooleanConstant(False)
+    return logic_or(*expressions)
+
+
 def logic_or(*expressions: ValueOrExpression) -> Expression:
+    """
+    Constructs a disjunction over the provided expressions.
+    """
     return _boolean_binary_expression(operators.BooleanOperator.OR, expressions)
 
 
+def logic_all(*expressions: ValueOrExpression) -> Expression:
+    """
+    Constructs a conjunction over the provided expressions.
+
+    Returns :attr:`~momba.model.expressions.TRUE` if there are no expressions.
+    """
+    if len(expressions) == 0:
+        return BooleanConstant(True)
+    return logic_and(*expressions)
+
+
 def logic_and(*expressions: ValueOrExpression) -> Expression:
+    """
+    Constructs a conjunction over the provided expressions.
+    """
     return _boolean_binary_expression(operators.BooleanOperator.AND, expressions)
 
 
 def logic_xor(*expressions: ValueOrExpression) -> Expression:
+    """
+    Constructs an exclusive disjunction over the provided expressions.
+    """
     return _boolean_binary_expression(operators.BooleanOperator.XOR, expressions)
 
 
 def logic_implies(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs a logical implication.
+    """
     return Boolean(
         operators.BooleanOperator.IMPLY, ensure_expr(left), ensure_expr(right)
     )
 
 
 def logic_equiv(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs a logical equivalence.
+    """
     return Boolean(
         operators.BooleanOperator.EQUIV, ensure_expr(left), ensure_expr(right)
     )
 
 
 def equals(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs an *equality* expression.
+    """
     return Equality(
         operators.EqualityOperator.EQ, ensure_expr(left), ensure_expr(right)
     )
 
 
 def not_equals(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs an *inequality* expression.
+    """
     return Equality(
         operators.EqualityOperator.NEQ, ensure_expr(left), ensure_expr(right)
     )
 
 
-def less_than(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+def less(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs a *less than* expression.
+    """
     return Comparison(
         operators.ComparisonOperator.LT, ensure_expr(left), ensure_expr(right)
     )
 
 
-def less_or_equal_than(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+def less_or_equal(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs a *less than or equal to* expression.
+    """
     return Comparison(
         operators.ComparisonOperator.LE, ensure_expr(left), ensure_expr(right)
     )
 
 
-def greater_than(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
-    return Comparison(
-        operators.ComparisonOperator.GE, ensure_expr(left), ensure_expr(right)
-    )
-
-
-def greater_or_equal_than(
-    left: ValueOrExpression, right: ValueOrExpression
-) -> Expression:
+def greater(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs a *greater than* expression.
+    """
     return Comparison(
         operators.ComparisonOperator.GT, ensure_expr(left), ensure_expr(right)
     )
 
 
+def greater_or_equal(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs a *greater than or equal to* expression.
+    """
+    return Comparison(
+        operators.ComparisonOperator.GE, ensure_expr(left), ensure_expr(right)
+    )
+
+
 def add(left: ValueOrExpression, right: ValueOrExpression) -> Expression:
+    """
+    Constructs an arithmetic addition expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.ADD, ensure_expr(left), ensure_expr(right)
     )
 
 
 def sub(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs an arithmetic substraction expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.SUB, ensure_expr(left), ensure_expr(right)
     )
 
 
 def mul(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs an arithmetic multiplication expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.MUL, ensure_expr(left), ensure_expr(right)
     )
 
 
 def mod(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs an euclidean remainder expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.MOD, ensure_expr(left), ensure_expr(right)
     )
 
 
 def real_div(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs a real-division expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.REAL_DIV,
         ensure_expr(left),
@@ -568,30 +965,45 @@ def real_div(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpress
 
 
 def log(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs a logarithm expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.LOG, ensure_expr(left), ensure_expr(right)
     )
 
 
 def power(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs a power expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.POW, ensure_expr(left), ensure_expr(right)
     )
 
 
 def minimum(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs a minimum expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.MIN, ensure_expr(left), ensure_expr(right)
     )
 
 
 def maximum(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs a maximum expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.MAX, ensure_expr(left), ensure_expr(right)
     )
 
 
 def floor_div(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpression:
+    """
+    Constructs an euclidean division expression.
+    """
     return ArithmeticBinary(
         operators.ArithmeticBinaryOperator.FLOOR_DIV,
         ensure_expr(left),
@@ -602,31 +1014,45 @@ def floor_div(left: ValueOrExpression, right: ValueOrExpression) -> BinaryExpres
 UnaryConstructor = t.Callable[[ValueOrExpression], Expression]
 
 
-def logic_not(operand: ValueOrExpression) -> Expression:
-    return Not(operators.NotOperator.NOT, ensure_expr(operand))
-
-
 def floor(operand: ValueOrExpression) -> Expression:
+    """
+    Constructs a floor expression.
+    """
     return ArithmeticUnary(
         operators.ArithmeticUnaryOperator.FLOOR, ensure_expr(operand)
     )
 
 
 def ceil(operand: ValueOrExpression) -> Expression:
+    """
+    Constructs a ceil expression.
+    """
     return ArithmeticUnary(operators.ArithmeticUnaryOperator.CEIL, ensure_expr(operand))
 
 
 def absolute(operand: ValueOrExpression) -> Expression:
+    """
+    Constructs an absolute value expression.
+    """
     return ArithmeticUnary(operators.ArithmeticUnaryOperator.ABS, ensure_expr(operand))
 
 
 def sgn(operand: ValueOrExpression) -> Expression:
+    """
+    Constructs a sign expression.
+    """
     return ArithmeticUnary(operators.ArithmeticUnaryOperator.SGN, ensure_expr(operand))
 
 
 def trunc(operand: ValueOrExpression) -> Expression:
+    """
+    Constructs a truncate expression.
+    """
     return ArithmeticUnary(operators.ArithmeticUnaryOperator.TRC, ensure_expr(operand))
 
 
 def name(identifier: str) -> Name:
+    """
+    Constructs a name expression.
+    """
     return Name(identifier)
